package com.cht.tas.callout;

import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.security.SecureRandom;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Component
public class MqttTas {
    private final Logger Log = LoggerFactory.getLogger(MqttTas.class);

    @Value("${mqtt.phone-conn.brokerUrl}")
    private String brokerUrl = "";

    @Value("${mqtt.phone-conn.apiKey}")
    private String apiKey = "";

    @Autowired
    private IVRMqtt IVRMqtt;

    private static MqttClient mClient;
    private Random random = new SecureRandom();
    private ScheduledExecutorService reconnectMQTTThread = Executors.newSingleThreadScheduledExecutor();

    @PostConstruct
    public void constructor() {
        try {
            Log.debug("MQTT TAS start");
            String clientId = MqttClient.generateClientId();;

            mClient = new MqttClient(brokerUrl, clientId, new MemoryPersistence());
            mClient.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    Log.debug("Connection to MQTT broker lost! : ", cause);
                    int retryDelayTime = 1 + random.nextInt(10);
                    Log.debug("schedule retry connect to Mqtt Iot Server in {} s", retryDelayTime);
                    reconnectMQTTThread.schedule(() -> connectMQTTPhoneServer(), retryDelayTime, TimeUnit.SECONDS);
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) {
                    Log.debug("recv topic:{}", topic);

                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    //Log.debug("deliveryComplete() is called");
                }
            });

            connectMQTTPhoneServer();

        } catch (MqttException e) {
            Log.debug("MqttException:" + e.getMessage() + ", " + e.getReasonCode());
        }
    }


    private void connectMQTTPhoneServer() {
        MqttConnectOptions connOpts = new MqttConnectOptions();
        connOpts.setPassword(apiKey.toCharArray());
        connOpts.setUserName(apiKey);
        connOpts.setCleanSession(true);
        try {
            mClient.connect(connOpts);
            IVRMqtt.setSubscribeAllTopic();
            Log.debug("Mqtt Phone Connected");
        } catch (MqttException e) {
            Log.error("MQTT Exception message: {}, reason code: {}", e.getMessage(), e.getReasonCode());
            int retryDelayTime = 30 + random.nextInt(10);
            Log.debug("schedule retry connect to Mqtt Iot Server in {} s", retryDelayTime);
            reconnectMQTTThread.schedule(() -> connectMQTTPhoneServer(), retryDelayTime, TimeUnit.SECONDS);
        }
    }

    @PreDestroy
    public void destroy() {
        if (mClient != null) {
            try {
                Log.debug("destroy");
                mClient.disconnect();
            } catch (MqttException e) {
                e.printStackTrace();
            }
        }
    }

    static void setSubscribeTopicResp(String topic, IMqttMessageListener messageListener) {
        try {
            mClient.subscribe(topic, messageListener);
        } catch (MqttException e) {
            e.printStackTrace();
        }
    }

    static void publish(String topic, MqttMessage message) {
        try {
            mClient.publish(topic, message);
        } catch (MqttException e) {
            e.printStackTrace();
        }

    }

}
